@@ -18,4 +18,7 @@ pub use crate::AsyncThread as LuaAsyncThread;
 
 #[cfg(feature = "serialize")]
 #[doc(inline)]
-pub use crate::{LuaSerdeExt, SerializeOptions as LuaSerializeOptions};
+pub use crate::{
+    DeserializeOptions as LuaDeserializeOptions, LuaSerdeExt,
+    SerializeOptions as LuaSerializeOptions,
+};
