@@ -38,6 +38,9 @@ fn test_gc_control() -> Result<()> {
     let lua = Lua::new();
     let globals = lua.globals();
 
+    #[cfg(feature = "lua54")]
+    assert_eq!(lua.gc_gen(0, 0), mlua::GCMode::Incremental);
+
     #[cfg(any(feature = "lua54", feature = "lua53", feature = "lua52"))]
     {
         assert!(lua.gc_is_running());
@@ -59,6 +62,9 @@ fn test_gc_control() -> Result<()> {
     lua.gc_collect()?;
     assert_eq!(Arc::strong_count(&rc), 1);
 
+    #[cfg(feature = "lua54")]
+    assert_eq!(lua.gc_inc(0, 0, 0), mlua::GCMode::Generational);
+
     Ok(())
 }
 
